<?php

namespace Swissup\Recaptcha\Plugin\Checkout\Api;

use Magento\Captcha\Helper\Data;
use Magento\Checkout\Model\Session;
use Magento\Quote\Api\Data\PaymentInterface;
use Magento\Framework\Exception\CouldNotSaveException;
use Psr\Log\LoggerInterface;
use Swissup\Recaptcha\Helper\Data as DataHelper;
use Swissup\Recaptcha\Helper\IpAllow;

abstract class AbstractValidation
{
    /**
     * @var string
     */
    protected $initiator = 'Swissup_Recaptcha';

    /**
     * @var string
     */
    protected $formId = '';

    /**
     * @var Data
     */
    protected $helper;

    /**
     * @var Session
     */
    protected $session;
    /**
     * @var IpAllow
     */
    private $ipAllowHelper;
    /**
     * @var LoggerInterface
     */
    private $logger;
    /**
     * @var DataHelper
     */
    private $dataHelper;

    /**
     * @param Data $helper
     * @param Session $session
     * @param IpAllow $ipAllowHelper
     * @param LoggerInterface $logger
     */
    public function __construct(
        Data    $helper,
        Session $session,
        IpAllow $ipAllowHelper,
        LoggerInterface $logger,
        DataHelper $dataHelper
    ) {
        $this->helper = $helper;
        $this->session = $session;
        $this->ipAllowHelper = $ipAllowHelper;
        $this->logger = $logger;
        $this->dataHelper = $dataHelper;
    }

    /**
     * @param  PaymentInterface $paymentMethod
     * @throws CouldNotSaveException
     */
    protected function validateCaptcha(PaymentInterface $paymentMethod)
    {
        $captcha = $this->helper->getCaptcha($this->formId);
        $this->logger->info("mod2-checkout-recaptcha-IP-data value: ".$this->ipAllowHelper->getCurrentIp());
        $this->logger->info("mod2-checkout-recaptcha-IP-data check result: ".$this->ipAllowHelper->isIpAllowed());
        if ($captcha->isRequired() && !$this->dataHelper->isPaymentMethodToExclude($paymentMethod->getMethod()) &&
            !$this->ipAllowHelper->isIpAllowed() && !$this->isSuccessValidation()) {
            $gResponse = $paymentMethod->getExtensionAttributes() === null
                ? ''
                : $paymentMethod->getExtensionAttributes()->getSwissupRecaptchaResponse();
            $response = $captcha->verify($gResponse);
            if (!$response->isSuccess()) {
                $errors = implode(', ', $response->getErrorCodes());
                $this->logger->info("The order wasn't placed. Invalid recaptcha. Error-code: {$errors}. Module: {$this->initiator}");
                throw new CouldNotSaveException(
                    __("The order wasn't placed. Invalid recaptcha. [error-code: %1]", $errors, $this->initiator)
                );
            }

            $this->setSuccessValidation(true);
        }
    }

    /**
     * Is captcha already validated and is success.
     *
     * @return boolean
     */
    private function isSuccessValidation()
    {
        $endTime = $this->session->getData("swissup_recaptcha/{$this->formId}/end_time");
        $isValid = !!$this->session->getData("swissup_recaptcha/{$this->formId}/is_valid");
        if ($endTime >= time()) {
            return $isValid;
        }

        return false;
    }

    /**
     * Set success validation flag.
     *
     * @param boolean $isValid
     */
    private function setSuccessValidation($isValid)
    {
        $minutes = 2; //Each reCAPTCHA user response token is valid for two minutes,
        // and can only be verified once to prevent replay attacks.
        // https://developers.google.com/recaptcha/docs/verify#token_restrictions
        $this->session->setData('swissup_recaptcha', [
            $this->formId => [
                'end_time' => time() + $minutes * 60,
                'is_valid' => $isValid
            ]
        ]);
    }
}
